;8-   DS18B20
; 3 
;    5.02.2012
;AVR      ATmega32
; 	8
; PortC.4 -> LCD RS (register select)
; PortC.5 -> LCd E (Enable)
; PortC.0 ... PortC.3 -> LCD data.4 ... data.7

.include "m32def.inc"

;**********************************************
;*                              *
;**********************************************

.def    RezL   = R6
.def    RezH   = R7
.def    Done        = r10           ;  GetROMs
.def    ROMCount    = r11      ;  
.def    SaveSREG    = r12        ;  
.def    BitNum      = r13          ;  ROMsrch
.def    LastDis     = r14            ;  ROMsrch
.def    LasDisCpy   = r15         ;  ROMsrch

.def    temp        = r16           ; 1
.def    temp1        = r17         ; 2
.def    temp2        = r23        ; 2
.def    lower    = r18
.def    upper    = r19
.def    temp3  = r20      
.def    Counter      = r21
.def    DelayVal    = r22
.def    number  = r24
;**********************************************
;* DS1820                        * 
;**********************************************
.equ    ReadROM     = 0x33   ;    . 
.equ    MatchROM   = 0x55  ;       
.equ    SkipROM      = 0xCC   ;    
.equ    SearchROM  = 0xF0   ;     
.equ    SearchAlarm = 0xEC   ;  Alarm 

.equ    CpyScratch   = 0x48  ;     
.equ    WrScratch     = 0x4E   ;     
.equ    RecallEE         = 0xB8   ;    TH  TL
.equ    ReadPS          = 0x84
.equ    ReadScratchpad=0xBE  ;     .
.equ    ConvertT        =0x44        ;   

.equ  LCD_RS=0                               ;PortC
.equ  LCD_RW=1
.equ  LCD_EN=2

;**********************************************
;* The following delay constants are based on * 
;* a 8MHZ clock (CLK)                        *
;* times are not exact.                       *
;**********************************************

.equ    d500us  = 0xB2        ;500us CLK/64
.equ    d100us  = 0x83    ;100us CLK/8
.equ    d44us   =  0xCA          ;44us  CLK/8
.equ    d15us   =  0x6B        ;15us  CLK/1
.equ    d2us    =   0xEC        ;2us   CLK/1


.equ    TSTOP   =0               ; /c
.equ    TCK1    =1                ;/c  CK
.equ    TCK8    =2                ;/c  CK / 8
.equ    TCK64   =3               ;/c  CK / 64
.equ    TCK256  =4              ;/c  CK/ 256
.equ    TCK1024 =5             ;/c  CK / 1024

.equ    DQ  =PB0         ; PORTB   1-wire 

#define F_CPU 8000000
#ifndef F_CPU
	#error "F_CPU must be defined!"
#endif

#if F_CPU < 8000000
	#warning "F_CPU too low, possible wrong delay"
#endif

#define CYCLES_PER_US (F_CPU/1000000)   ;    
#define C4PUS (CYCLES_PER_US/4)	     ; 4   
#define DVUS(x) (C4PUS*x)	                     ;      , 
;   Wait4xCycles   

;**********************************************
; *  100    SRAM 
;**********************************************
.dseg
                
Buffer:              .byte 16      ;
TempLSB:         .byte 1        ;  
TempMSB:       .byte 1         ;     
THUB1:            .byte 1         ;TH-  1
TLUB2:             .byte 1         ;TL-  2
Config:             .byte 1         ; 
Res1:                .byte 1         ;
Res2:                .byte 1         ; 
Res3:                .byte 1         ;
CRC:                 .byte 1         ;CRC

ExtTemp:         .byte 1
romoff:           .byte 2         ;              
romdta:          .byte 8         ; ROM  
Sensor0:         .byte 8         ;DS1820  0 ID #
Sensor1:         .byte 8         ;DS1820  1 ID #
Sensor2:         .byte 8         ;DS1820  2 ID #
Sensor3:         .byte 8         ;DS1820  3 ID #
Sensor4:         .byte 8         ;DS1820  4 ID #
Sensor5:         .byte 8         ;DS1820  5 ID #
Sensor6:         .byte 8         ;DS1820  6 ID #
Sensor7:         .byte 8         ;DS1820  7 ID #
num:              .byte 1

.cseg
.org    0x00                  ;Reset Vector
        rjmp    Reset
   
 ;Interrupt vectors

.org    0x14                      ; / 0
        rjmp    Tim0int

Reset:
;   R0-R30
        ldi	ZL, 30		;    	
        clr	ZH		;      
        dec	ZL		;  
        st	Z, ZH		;    0
        brne	PC-2		;      
;  
RAM_Flush:
        ldi	ZL,Low(SRAM_START)
        ldi	ZH,High(SRAM_START)
        clr	temp
Flush:	
        st 	Z+,temp
        cpi	ZH,High(RAMEND)
        brne	Flush
        cpi	ZL,Low(RAMEND)
        brne	Flush
        clr	ZL
        clr	ZH
        ldi temp,high(RAMEND)         ;    RAM
        out SPH,temp                      
        ldi temp,low(RAMEND)
        out SPL,temp
        ldi temp,0xFF
        out  DDRC,temp
        ldi    temp,TSTOP                 ;0  (  )
        out    TCCR0,temp              
        ldi    temp,(1<<OCIE0)         ;     / T0
        out    TIMSK,temp
        sei                                        ;  
        rcall LCD_init
        rcall Text_OUT
        rcall  GetROMS                    ; ROMS 
;********************************************** 
;*                                     *
;*   1-Wire        
;**********************************************
Main: 
        cli                                           ;                                         
        ldi      number,0x00
Main1:
        cp     number,ROMCount        ;,   <=  
        brcc  End_Main                       ; 
        mov temp,number
        sts    num,number 
        rcall  GetRegs                        ;   
        rcall  CvtExt  
        rcall  TempCvt                        ;   ASCII
        rcall  Display      ;  
        rcall WaitMiliseconds
End_Main:
        inc   number
        cpi number,8
        brne  Main1
        clr number 
        sei                                         ; 
        rjmp Main
      
;**********************************************
;* DS18B20                            *
;********************************************************************************
; 
;     1-Wire    Reset.
;       C  SGER   
; : 0 -     , 1 -  
;********************************************************************************
OWReset: 
        sbi    DDRB,DQ              ;   0.   
        cbi    PORTB,DQ            ;
        ldi    temp3,TCK64         ; 
        ldi    DelayVal,d500us
        rcall  Delay                     ; 500 
        cbi    DDRB,DQ              ;   
        sbi    PORTB,DQ             ;
        ldi    temp3,TCK8           ; 
        ldi    DelayVal,d100us
        rcall  Delay                    ; 100  
        clc                                 ;   0   
        sbis   PINB,DQ              ; ,  0 =1
        sec                                ;  , 0 =0 
        ldi    temp3,TCK64         ; 
        ldi    DelayVal,d500us
        rcall  Delay                   ; 500 
        ret
;********************************************************************
;  1 
;    1    1-Wire.
;        
;********************************************************************
OWReadBit:
        ldi    temp3,TCK1      ; 
        ldi    DelayVal,d2us        ;
        sbi    DDRB,DQ              ;   0  2 
        cbi    PORTB,DQ            ;
        rcall  Delay
        cbi    DDRB,DQ             ;   
        sbi    PORTB,DQ            ;
        ldi    temp3,TCK1      ; 
        ldi    DelayVal,d15us      
        rcall  Delay                    ; 15  
        clc                                ;   0
        sbic   PINB,DQ              ; ,  0 =0
        sec                                ;1-Wire ,   
        ldi    temp3,TCK8      ; 
        ldi    DelayVal,d44us     ; 44    
        rcall  Delay
        ret
;**********************************************************************************
;  1 
;    1    1-Wire.
;          
;**********************************************************************************
OWWriteBit:
        sbi    DDRB,DQ            ;DDRB bit 0 = 1 = 
        cbi    PORTB,DQ     
        ldi    temp3,TCK1     ; 
        ldi    DelayVal,d15us     ; 15  
        rcall  Delay           
        brcc   Put0                   ;   = 0,    Put0
        cbi    DDRB,DQ            ;   
        sbi    PORTB,DQ     ;
Put0:
        ldi    temp3,TCK8     ; 
        ldi    DelayVal,d44us     ;
        rcall  Delay                   ; 44  
        cbi    DDRB,DQ            ;   
        sbi    PORTB,DQ           ;
        ldi    temp3,TCK1     ; 
        ldi    DelayVal,d2us       ;
        rcall  Delay                   ; 2 
        ret
;******************************************************
;  1 
;    1    1-Wire.
;     temp
;*****************************************************
OWReadByte:
        push	Counter	         ;  -
        ldi    Counter,8          ; 
        clr    temp
NxtBit:
        rcall  OWReadBit
        ror    temp                ;   r16     
        dec    Counter
        brne   NxtBit             ; -  
        pop	Counter	         ;  -
        ret

;*****************************************************************
;  1 
;    1    1-Wire.
;*****************************************************************
OWWriteByte:
        push	Counter	            ;  -
        ldi    Counter,8             ; 
ShiftB:
        lsr    temp                    ;     C
        rcall  OWWriteBit
NextB:
        dec    Counter
        brne   ShiftB                ; -  
        pop	Counter	           ; -
        ret
;**********************************************
;*     DS              
;**********************************************
ROM1st:
        ldi    temp,0x41               ;  Rom  (1- ).
        mov    LastDis,temp
        rcall  ROMSrch
        ret
ROMSrch:
        push   XL
        push   XH
        rcall  OWReset                    ; 1-wire .
        brcs   r_look
        rjmp   NoDev
r_look:
        ldi    temp,0x40                    ;  64   ROM.
        mov    BitNum,temp
        ldi    Counter,8                    ; 
        mov    LasDisCpy, LastDis   ;     .
        ldi    XL,low(romdta)
        ldi    XH,high(romdta)         ;  1-   ROM
        sts    romoff,XL                  ;  rombuff.
        sts    romoff+1,XH
        ldi    temp,SearchROM      ;   ROM  temp
        rcall  OWWriteByte           ;   ROM    .
NxtRBit:
        clr    temp
        clr    temp1
        rcall  OWReadBit               ;1-  time-slot.
        rol    temp1                      ; temp1, bit0   
        rcall  OWReadBit              ;2-  time-slot.
        rol    temp
        cpi    temp1,1                   ;    1?
        brne   no_or                     ;,   
        ori    temp,2                     ;  1  temp
no_or:
        cpi    temp,3                     ;.,    .
        breq   NoDev                    ; ,  .
        cpi    temp,0                      ;    .
        brne   nosave                    ;,   .
        cp     LasDisCpy,BitNum    ;     .        
        brne   lwhere                     ;    .
        ldi    temp,1                       ;       0.
        rjmp   wrtBit                      ; ,  1.
lwhere:
        brcs   sLstBit                     ;   .
        clr    temp                         ;    .
        rjmp   savedis                    ; 0  .
sLstBit:
        lds    XL,romoff                 ;    ROM
        lds    XH,romoff+1  
        ld     temp,X                      ; ,     temp.
        andi   temp,1                    ; lsb temp.
        cpi    temp,1                     ;.,    1.
        breq   wrtBit                     ;      .
savedis: 
        mov    LastDis,BitNum      ;     x.
        rjmp   wrtBit                     ;   time slot
nosave:
        ldi    temp1,1
        eor    temp,temp1             ;  lsb temp.
wrtBit:
        andi   temp,1                    ; lsb temp.
        mov    temp1,temp
        lsr    temp1
        rcall  OWWriteBit             ; lsb temp to 1-wire .
        lds    XL,romoff 
        lds    XH,romoff+1          ;     ROM.
        ld     temp,X                    ;  val rombyte.
        ror    temp                      ;  ROM  MSB rombyte.
        st     X,temp                     ;          
        dec    Counter                 ; 
        brne   bnfull                    ;,     .
        adiw   XH:XL,1                 ; x
        sts    romoff,XL               ;  rombuff.
        sts    romoff+1,XH
        ldi    Counter,8                ; 
bnfull:
        dec    BitNum                  ;  64  
        brne   NxtRBit                 ;  rom .
        cp     LasDisCpy,LastDis   ;    
        brne   MoDows               ;   temp,
        ldi    temp,0x41                ; ROM    
        mov    LastDis,temp        ;  .
        ldi    temp,1                    ;,       
        rjmp   lastdow                ; clra   .
MoDows:
        clr    temp                      ;,             
lastdow:
        sec                                 ; .
        rjmp   DevFnd                ;   
NoDev:
        clc                                 ; 
DevFnd:
        pop    XH
        pop    XL
        ret
;********************************************** 
;* GetROMS                                    *
; *   8 DS1820s,       *
; * 64  ROM       *
;**********************************************
GetROMS:
        clr    temp
        clr    ROMCount              ; ROMCount
        clr    Done                       ;  
        ldi    YH,high(romdta)
        ldi    YL,low(romdta)         ;Y    romdta
ClrOut:
        st     Y+,temp                   ; ROM
        cpi    YL,low(Sensor7+8)
        brcs   ClrOut                    ;,  =1 
LookROM:
        rcall  OWReset                  ;Reset 1-wire bus
; ***,     ROM,   
        brcc   SetExit         ; ,     
; ***,      ROM,   
;       brcc   SetExit2       ; ,     

        rcall  ROM1st          ;  ROM
        mov    lower,temp
        brcc   SetExit         ; temp = 0   
        ldi    XH,high(Sensor0)
        ldi    XL,low(Sensor0)   ;     
        rjmp   MR0                  ;   
LR1:
        rcall  ROMSrch           ;  
        mov    lower,temp
        breq   MR0                 ; temp = 0   
        inc    Done
MR0:
        ldi    YH,high(romdta+8)
        ldi    YL,low(romdta+8)    ;Y    ROM
MoveROM:
        ld     temp,-Y                  ;   romdta
        st     X+,temp                 ; SensorX
        cpi    YL,low(romdta)       ;   8 
        brne   MoveROM            ;   ROM
        inc    ROMCount             ;   ROM
        tst    lower                 ; ,     
        breq   MR2
        inc    Done
MR2:
        tst    Done
        brne   SetExit
        cpi    XL,low(Sensor7+8)  ;    ROM
        brcs   LR1                        ; 
SetExit:
        ret
;********************************************** 
;* ROMatch                                    *
;*   MatchROM,     *
;* 8-   ROM                          * 
;**********************************************
ROMatch:
        push  Counter
        lds    XL,romoff                     ;  ROM 
        lds    XH,romoff+1
        adiw   XH:XL,8                       ;  MSB                             
        ldi    Counter,8
        ldi    temp,MatchROM          ; Match ROM 
        rcall  OWWriteByte               ; 
SndROM:
        ld     temp,-X                        ;   ROM
        rcall  OWWriteByte
        dec    Counter
        brne   SndROM 
        pop   Counter 
        ret     
;**********************************************
;*    Timer0         
;**********************************************
Tim0int:
        cli                                          ;  
        set                                        ;  T 
        ldi     temp3,TSTOP           ; Timer 0
        out     TCCR0,temp3
        sei                                        ; 
        reti                    
;**********************************************
;*       CLK      
;* DelayVal  .          
;*  upper              
;**********************************************
Delay:
        in      SaveSREG,SREG            ;  
        out     TCNT0,DelayVal
        clt                                         ;  T
        out     TCCR0,temp3         ; 
Dwait:  
        brtc    Dwait                         ;,  T 
        out     SREG,SaveSREG
        ret
;********************************************** 
;* GetRegs                                    *
; *      DS18B20
; *    temp,temp1 
;**********************************************
GetRegs:
        sei
        rcall  OWReset
        ldi    XL,low(Sensor0)
        ldi    XH,high(Sensor0)
GR0:
        mov    temp1,temp           ;  #  temp1
        lsl    temp1                       ;temp1 x 2
        lsl    temp1                       ;temp1 x 4
        lsl    temp1                       ;temp1 x 8
        adc    XL,temp1
        brcc   GR1   
        inc    XH                           ;  ,     XH    
GR1:
        sts    romoff,XL
        sts    romoff+1,XH            ; 
        rcall  ROMatch                 ;   
        ldi    temp,ConvertT          ;  Temperature command
        rcall  OWWriteByte  
WaiTemp:
        rcall  OWReadBit                ;                
        brcc   WaiTemp                  ;DS18S20        
        rcall  OWReset                    ; 
        rcall  ROMatch                   ;  
        ldi   temp,ReadScratchpad  ; scratchpad command
        rcall  OWWriteByte
        ldi    XL,low(TempLSB)        ; Scratchpad 
        ldi    XH,high(TempMSB)  
RdMem:                                     ;   1820 scratchpad memory  
        rcall  OWReadByte             ; 
        st     X+,temp                     ;lower byte
        cpi    XL,low(TempLSB+16)   ; 9 
        brcs   RdMem
        lds    temp1,TempMSB         ;   temp TempMSB
        lds    temp,TempLSB           ;,  -  temp
        sts    TempLSB,temp          ;   
        sts    TempMSB,temp1          ;   
        rcall  ShowData 
        cli
        ret 
;********************************************** 
;* ShowData                                   
;* CRC    
;*  CRC
;**********************************************
ShowData:
        push Counter
        clr temp1
        ldi    XL,low(TempLSB)              ;X    
        ldi    XH,high(TempLSB)
        ldi    Counter,8
Read_CRC:
        ld     temp,X+
        rcall  CRC8                        
        dec    Counter
        brne   Read_CRC 
        lds  temp,CRC
        eor temp,temp1
        brne  CRC_Error
        rjmp   CRC_Accuracy
CRC_Error:                         ;   
        ldi  Temp,0xCC          ;   LCD   
        rcall LCD_command
        ldi temp,'N'  
        rcall LCD_putchar 
        ldi temp,'O'  
        rcall LCD_putchar
         ldi temp,'!'  
        rcall LCD_putchar
        rjmp   End_ShowData
CRC_Accuracy:                 ;    
        ldi  Temp,0xCC         ;   LCD   
        rcall LCD_command
        ldi temp,'O'  
        rcall LCD_putchar 
        ldi temp,'K'  
        rcall LCD_putchar
        ldi temp,'!'  
        rcall LCD_putchar
End_ShowData:                    
        pop  Counter
        ret
;********************************************************
; .. temp -  
; temp1 -  CRC
;********************************************************
CRC8:
       push  Counter
       push  temp3
       ldi   Counter,8
CRC_Loop:
       mov   temp3,temp
       eor   temp3,temp1
       lsr   temp3
       brcc  CRC_Zero
       ldi   temp3,0x18
       eor   temp1,temp3
CRC_Zero:
       ror   temp1
       clc
       sbrc  temp,0
       sec
       ror   temp
       dec   Counter
       brne  CRC_Loop
       pop   temp3
       pop   Counter
       ret
;*********************************************************************************
;CvtExt     
;  10    16
;********************************************************************************
CvtExt:
         push  Counter
         push  temp
         push  temp1 
         lds     upper,TempMSB         ;   temp TempMSB
         lds     lower,TempLSB           ;   temp1 TempLSB 
         sbrs upper,7                        ;  
         rjmp  Mul16x8
         subi lower,2
         com lower
         com upper
         rjmp Mul16x8
Mul16x8:                     
         ldi temp,10                           ;                
         mul	lower, temp	     ; AL * BL
         movw  RezH:RezL, r1:r0       
         mul	upper, temp	     ; AH * BH
         add	RezH, r0
div16:                                            ;  16  
        clr       Counter
div16x8:
        lsr       RezH
        ror      RezL
        inc     Counter
        cpi      Counter,4
        brne    div16x8 
        mov   lower,RezL
        mov upper,RezH
        sts    TempLSB,upper              ;  
        sts    ExtTemp,lower                ;  
        pop  Counter 
        pop temp
        pop temp1 
        ret
;**********************************************
;* TempCvt                                    
;*  ASCII                           
;**********************************************  
TempCvt:
        ldi    YL, low(Buffer)
        ldi    YH,high(Buffer)
        lds    temp,TempMSB        ;    
        tst    temp                         ;  
        brmi   TS0                         ;  
        ldi    temp,'+'
        st     Y+,temp                   ; +  
        rjmp   Cvt2BCD         
TS0:
        ldi    temp,'-'
        st     Y+,temp                   ; +  
Cvt2BCD:
        lds    temp1,TempLSB       ;   
        lds    temp,ExtTemp          ; BCD
        rcall  BIN2BCD16
        mov    temp,r14               ;   4 
        swap   temp
        rcall  Put1Asc                  ; 4  ASCII
        mov    temp,r14
        rcall  Put1Asc
        mov    temp,r13
        swap   temp
        rcall  Put1Asc
        ldi    temp,'.'                     ;    
        st     Y+,temp
        mov    temp,r13
        rcall  Put1Asc
        ldi    temp,'C'  
        st     Y+,temp
        clr    temp
        st     Y,temp
        ret
;**********************************************
; * Bin2BCD16
; *     16   
; *  5    BCD
; * ,    temp1:temp *
; *  MSB  upper, LSB  lower *
; *   : *
; * r13  1  0  BCD
; * r14  3  2   BCD
; * r15  4  BCD
; *   temp-  *
; * temp1-  *
; * Counter
; * upper 
;**********************************************  

.equ	AtBCD0	=13		; r13
.equ	AtBCD2	=15		; r14

bin2BCD16:
        push Counter
        ldi    counter,16                                ; 	
        clr    r15                                           ;  (3 )
        clr    r14		
        clr    r13		
        clr    ZH                                           ; ZH
bBCDx_1:
        lsl    temp                                        ;  
        rol    temp1                                     ;  
        rol    r13
        rol    r14
        rol    r15
        dec    counter                           ; 
        brne   bBCDx_2                        ;   ,
        pop  Counter
        ret                                            ;
bBCDx_2:
        ldi    r30,AtBCD2+1                 ;Z ,   MSB + 1
bBCDx_3:
        ld     upper,-Z                          ; (Z)  
        subi   upper,-0x03                   ; 0x03
        sbrc   upper,3                         ;  3 , 
        st     Z,upper                           ;   
        ld     upper,Z                           ; (Z)
        subi   upper,-0x30                  ; 0x30
        sbrc   upper,7                         ;  7 , 
        st     Z,upper                           ;   
        cpi    ZL,AtBCD0                     ;  ?
        brne   bBCDx_3                      ; ,  
        rjmp   bBCDx_1		
; **********************************************
; *   LSB temp  ASCII *
; **********************************************
Put1Asc:
        andi   temp,0x0F
        cpi    temp,0x0A
        brmi   PTASC1
        ldi    temp1,7
        add    temp,temp1
PTASC1:
        ldi    temp1,0x30
        add    temp,temp1
        st     Y+,temp
        ret
;***********************************************************************
; LCD
;***********************************************************************
LCD_init: 
       push temp
       ldi temp,0x33
       rcall Lcd_command        ; LCD  4-.
       ldi    XH,HIGH(DVUS(2000))
       ldi    XL,LOW(DVUS(2000))
       rcall  Wait4xCycles                  ; 2ms
       ldi temp,0x32                ; LCD  4-. 
       rcall Lcd_command             
       ldi    XH,HIGH(DVUS(2000))
       ldi    XL,LOW(DVUS(2000))
       rcall  Wait4xCycles                  ; 2ms
       ldi temp,0x28                ; LCD 2 , 57
       rcall Lcd_command
       ldi    XH,HIGH(DVUS(2000))
       ldi    XL,LOW(DVUS(2000))
       rcall  Wait4xCycles                  ; 2ms
       ldi temp,0x0C	      ;. ,  
       rcall Lcd_command
       ldi temp,0x01               ; LCD
       rcall Lcd_command      
       ldi    XH,HIGH(DVUS(2000))
       ldi    XL,LOW(DVUS(2000))
       rcall  Wait4xCycles                  ; 2ms
       ldi temp,0x06                ;  
       rcall Lcd_command
       pop   temp
       ret
;***********************************************************************
;  LCD
;***********************************************************************
Lcd_command:
       push temp1
       push temp2
       push temp         
       mov  temp1,temp
       andi  temp1,0xF0
       in   temp2,PORTC
       andi  temp2,0x0F
       or     temp2,temp1
       out   PORTC,temp2
       cbi    PORTC,LCD_RS 
       cbi    PORTC,LCD_RW 
       sbi    PORTC,LCD_EN
       ldi    XH,HIGH(DVUS(1))
       ldi    XL,LOW(DVUS(1))
       rcall  Wait4xCycles 
       cbi    PORTC,LCD_EN  
       ldi    XH,HIGH(DVUS(100))
       ldi    XL,LOW(DVUS(100))
       rcall  Wait4xCycles 

       mov  temp1,temp
       swap temp1 
       andi  temp1,0xF0 
       in   temp2,PORTC
       andi  temp2,0x0F
       or     temp2,temp1
       out   PORTC,temp2
       sbi    PORTC,LCD_EN
       ldi    XH,HIGH(DVUS(1))
       ldi    XL,LOW(DVUS(1))
       rcall  Wait4xCycles
       cbi    PORTC,LCD_EN
       ldi    XH,HIGH(DVUS(100))
       ldi    XL,LOW(DVUS(100))
       rcall  Wait4xCycles 
       pop temp1
       pop temp2
       pop temp
       ret
;***********************************************************************
; 
;***********************************************************************
Lcd_putchar:
      push temp
      push temp1
      push temp2
      mov  temp1,temp
      andi  temp1,0xF0
      in   temp2,PORTC
      andi  temp2,0x0F
      or     temp2,temp1
      out   PORTC,temp2
      sbi    PORTC,LCD_RS 
      cbi    PORTC,LCD_RW 
      sbi    PORTC,LCD_EN
      ldi    XH,HIGH(DVUS(1))
      ldi    XL,LOW(DVUS(1))
      rcall  Wait4xCycles
      cbi    PORTC,LCD_EN  
   
      mov  temp1,temp
      swap temp1 
      andi  temp1,0xF0 
      in   temp2,PORTC
      andi  temp2,0x0F
      or     temp2,temp1
      out   PORTC,temp2
      sbi    PORTC,LCD_EN
      ldi    XH,HIGH(DVUS(1))
      ldi    XL,LOW(DVUS(1))
      rcall  Wait4xCycles
      cbi    PORTC,LCD_EN
      ldi    XH,HIGH(DVUS(100))
      ldi    XL,LOW(DVUS(100))
      rcall  Wait4xCycles 
      pop temp1
      pop temp2
      pop temp
      ret

;****************************************************
;  
;****************************************************
Display:
        push Temp
        push  counter
        ldi  Temp,0xC5         ;   LCD   
        rcall LCD_command
        lds   temp,num
        ori temp,0x30
        rcall LCD_putchar
        ldi Temp,0x85            ;  
        rcall LCD_command
        ldi    YL, low(Buffer)
        ldi    YH,high(Buffer)
        ldi    counter,8
Loop2:
        ld    temp,Y+
        rcall LCD_putchar
        dec  counter
        brne  Loop2
        ldi    XH,HIGH(DVUS(5000))
        ldi    XL,LOW(DVUS(5000))
        rcall  Wait4xCycles
        pop   temp
        pop   counter
        ret 
;****************************************************
;   1
;****************************************************
Text_OUT:
       clr  counter
       ldi Temp,0x80                 ;  
       rcall LCD_command	
       ldi ZL,Low(Text01*2) 
       ldi ZH,High(Text01*2)
       ldi counter,6
       rcall  loop1
  
       ldi Temp,0xC0                ;  
       rcall LCD_command
       ldi ZL,Low(Text02*2) 
       ldi ZH,High(Text02*2)
       ldi counter,6
       rcall  loop1

       ldi Temp,0xC8               ;  
       rcall LCD_command
       ldi ZL,Low(Text03*2) 
       ldi ZH,High(Text03*2)
       ldi counter,4
       rcall  loop1 
       ret
loop1:
       lpm  temp,Z+
       rcall LCD_putchar
       dec  counter
       brne  loop1
       ret 
Text01:
      .db  0x54,0x65,0x6D,0x70,0x3D,0
Text02:
      .db 0x53,0x65,0x6E,0x73,0x3D,0
Text03:
      .db 0x43, 0x52, 0x43, 0
;--------------------------------------------------------------------
; Input : XH:XL - number of CPU cycles to wait (divided by four)
;--------------------------------------------------------------------
Wait4xCycles:
	sbiw	XH:XL, 1			; 2 
	brne	Wait4xCycles		; 1/2
	ret				; 4
;********************************************************************
;      
;  : temp =  
;********************************************************************
WaitMiliseconds:
	push 	temp  ;120
	ldi temp,100
WaitMsLoop1: 
	ldi    XH,HIGH(DVUS(600))
	ldi    XL,LOW(DVUS(600))
	rcall  Wait4xCycles
	ldi    XH,HIGH(DVUS(600))
	ldi    XL,LOW(DVUS(600))
	rcall  Wait4xCycles
	dec    temp
	brne   WaitMsLoop1
	pop    temp
	ret